@@ -29,7 +29,7 @@ fn bench_constructor_with_custom_dict(c: &mut Criterion) {
 }
 
 fn bench_tokenize(c: &mut Criterion) {
-    let mut tokenizer = Tokenizer::new().unwrap();
+    let tokenizer = Tokenizer::new().unwrap();
     c.bench_function("bench-tokenize-wiki", |b| {
         b.iter(|| tokenizer.tokenize("検索エンジン（けんさくエンジン、英語: search engine）は、狭義にはインターネットに存在する情報（ウェブページ、ウェブサイト、画像ファイル、ネットニュースなど）を検索する機能およびそのプログラム。"))
     });
@@ -42,7 +42,7 @@ fn bench_tokenize_with_custom_dict(c: &mut Criterion) {
         mode: Mode::Normal,
         ..TokenizerConfig::default()
     };
-    let mut tokenizer = Tokenizer::with_config(config).unwrap();
+    let tokenizer = Tokenizer::with_config(config).unwrap();
     c.bench_function("bench-tokenize-custom-dict", |b| {
         b.iter(|| tokenizer.tokenize("東京スカイツリーの最寄り駅はとうきょうスカイツリー駅です"))
     });
@@ -52,7 +52,7 @@ fn bench_tokenize_long_text(c: &mut Criterion) {
     let mut large_file = BufReader::new(File::open("resources/bocchan.txt").unwrap());
     let mut large_text = String::new();
     let _size = large_file.read_to_string(&mut large_text).unwrap();
-    let mut tokenizer = Tokenizer::new().unwrap();
+    let tokenizer = Tokenizer::new().unwrap();
     // Using benchmark_group for changing sample_size
     let mut group = c.benchmark_group("Long text");
     group.sample_size(20);
